/*
 * ExactPatternMatch.h
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#ifndef _EXACT_PATTERN_MATCH_H_
#define _EXACT_PATTERN_MATCH_H_

#include <tree/Tree.h>
#include <tree/TreeFeatures.h>
#include <set>
#include <tree>
#include <core/multipleDispatch.hpp>
#include <alphabet/RankedSymbol.h>
#include <alphabet/Symbol.h>

namespace arbology {

namespace exact {

class ExactPatternMatch : public std::DoubleDispatch < ExactPatternMatch, std::set < unsigned >, const tree::TreeBase &, const tree::TreeBase & > {
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static std::set < unsigned > match ( const tree::Tree & subject, const tree::Tree & pattern );

	static std::set < unsigned > match ( const tree::UnrankedTree & subject, const tree::UnrankedPattern & pattern );

	static std::set < unsigned > match ( const tree::RankedTree & subject, const tree::RankedPattern & pattern );
	static std::set < unsigned > match ( const tree::RankedTree & subject, const tree::RankedNonlinearPattern & pattern );

	static std::set < unsigned > match ( const tree::PrefixRankedTree & subject, const tree::PrefixRankedPattern & pattern );
	static std::set < unsigned > match ( const tree::PrefixRankedTree & subject, const tree::PrefixRankedNonlinearPattern & pattern );

	static std::set < unsigned > match ( const tree::PrefixRankedBarTree & subject, const tree::PrefixRankedBarPattern & pattern );
	static std::set < unsigned > match ( const tree::PrefixRankedBarTree & subject, const tree::PrefixRankedBarNonlinearPattern & pattern );

private:
	static bool matchHelper ( const std::tree < alphabet::Symbol > & subject, const std::tree < alphabet::Symbol > & pattern, const alphabet::Symbol & subtreeVariable );
	static bool matchHelper ( const std::tree < alphabet::RankedSymbol > & subject, const std::tree < alphabet::RankedSymbol > & pattern, const alphabet::RankedSymbol & subtreeVariable );
	static bool matchHelper ( const std::tree < alphabet::RankedSymbol > & subject, const std::tree < alphabet::RankedSymbol > & pattern, const alphabet::RankedSymbol & subtreeVariable, const std::set < alphabet::RankedSymbol > & nonlinearVariables, const std::tree < alphabet::RankedSymbol > & repeats, std::map < alphabet::RankedSymbol, alphabet::Symbol > & variablesSetting );

	static void matchInternal ( unsigned & index, std::set < unsigned > & occ, const std::tree < alphabet::Symbol > & subject, const std::tree < alphabet::Symbol > & pattern, const alphabet::Symbol & subtreeVariable );
	static void matchInternal ( unsigned & index, std::set < unsigned > & occ, const std::tree < alphabet::RankedSymbol > & subject, const std::tree < alphabet::RankedSymbol > & pattern, const alphabet::RankedSymbol & subtreeVariable );
	static void matchInternal ( unsigned & index, std::set < unsigned > & occ, const std::tree < alphabet::RankedSymbol > & subject, const std::tree < alphabet::RankedSymbol > & pattern, const alphabet::RankedSymbol & subtreeVariable, const std::set < alphabet::RankedSymbol > & nonlinearVariables, const std::tree < alphabet::RankedSymbol > & subjectRepeats );

};

} /* namespace exact */

} /* namespace arbology */

#endif /* _EXACT_PATTERN_MATCH_H_ */
