/*
 * ExactSubtreeMatch.h
 *
 *  Created on: 9. 2. 2014
 *      Author: Jan Travnicek
 */

#ifndef _EXACT_SUBTREE_MATCH_H_
#define _EXACT_SUBTREE_MATCH_H_

#include <tree/Tree.h>
#include <tree/TreeFeatures.h>
#include <set>
#include <tree>
#include <core/multipleDispatch.hpp>
#include <alphabet/RankedSymbol.h>
#include <alphabet/Symbol.h>

namespace arbology {

namespace exact {

class ExactSubtreeMatch : public std::PromotingDoubleDispatch<ExactSubtreeMatch, std::set<unsigned>, const tree::TreeBase & > {
public:
	/**
	 * Performs conversion.
	 * @return left regular grammar equivalent to source automaton.
	 */
	static std::set<unsigned> match(const tree::Tree& subject, const tree::Tree& pattern);

	static std::set<unsigned> match(const tree::UnrankedTree& subject, const tree::UnrankedTree& pattern);
	static std::set<unsigned> match(const tree::RankedTree& subject, const tree::RankedTree& pattern);
	static std::set<unsigned> match(const tree::PrefixRankedTree& subject, const tree::PrefixRankedTree& pattern);
	static std::set<unsigned> match(const tree::PrefixRankedBarTree& subject, const tree::PrefixRankedBarTree& pattern);
private:
	static bool matchHelper(const std::tree < alphabet::Symbol > & subject, const std::tree < alphabet::Symbol > & pattern);
	static bool matchHelper(const std::tree < alphabet::RankedSymbol > & subject, const std::tree < alphabet::RankedSymbol > & pattern);

	static void matchInternal(unsigned& index, std::set<unsigned>& occ, const std::tree < alphabet::Symbol > & subject, const std::tree < alphabet::Symbol > & pattern);
	static void matchInternal(unsigned& index, std::set<unsigned>& occ, const std::tree < alphabet::RankedSymbol > & subject, const std::tree < alphabet::RankedSymbol > & pattern);

};

} /* namespace exact */

} /* namespace arbology */

#endif /* _EXACT_SUBTREE_MATCH_H_ */
