/*
 * Result.cpp
 *
 *  Created on: 9. 2. 2014
 *	  Author: Jan Travnicek
 */

#include "Result.h"
#include "Run.h"
#include <automaton/FSM/DFA.h>
#include <automaton/TA/DFTA.h>
#include <automaton/PDA/InputDrivenDPDA.h>
#include <automaton/PDA/VisiblyPushdownDPDA.h>
#include <automaton/PDA/RealTimeHeightDeterministicDPDA.h>
#include <automaton/PDA/DPDA.h>
#include <label/LabelSetLabel.h>

#include <deque>

namespace automaton {

namespace run {

label::Label Result::result ( const automaton::Automaton & automaton, const alib::Object & object, const label::Label & failLabel ) {
	return dispatch ( automaton.getData ( ), object.getData ( ), failLabel );
}

label::Label Result::result ( const automaton::DFA < > & automaton, const string::LinearString < > & string, const label::Label & failLabel ) {
	std::tuple < bool, label::Label, std::set < unsigned > > res = Run::calculateState ( automaton, string );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res );

	return failLabel;
}

auto ResultDFALinearString = Result::RegistratorWrapper < label::Label, automaton::DFA<>, string::LinearString < > > ( Result::result );

label::Label Result::result ( const automaton::DFTA & automaton, const tree::RankedTree & tree, const label::Label & failLabel ) {
	std::tuple < bool, label::Label, std::set < unsigned > > res = Run::calculateState ( automaton, tree );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res );

	return failLabel;
}

auto ResultDFTARankedTree = Result::RegistratorWrapper < label::Label, automaton::DFTA, tree::RankedTree > ( Result::result );

label::Label Result::result ( const automaton::InputDrivenDPDA < > & automaton, const string::LinearString < > & string, const label::Label & failLabel ) {
	std::tuple < bool, label::Label, std::set < unsigned >, std::deque < alphabet::Symbol > > res = Run::calculateState ( automaton, string );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res );

	return failLabel;
}

auto ResultInputDrivenDPDALinearString = Result::RegistratorWrapper < label::Label, automaton::InputDrivenDPDA < >, string::LinearString < > > ( Result::result );

label::Label Result::result ( const automaton::VisiblyPushdownDPDA < > & automaton, const string::LinearString < > & string, const label::Label & failLabel ) {
	std::tuple < bool, label::Label, std::set < unsigned >, std::deque < alphabet::Symbol > > res = Run::calculateState ( automaton, string );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res );

	return failLabel;
}

auto ResultVisiblyPushdownDPDALinearString = Result::RegistratorWrapper < label::Label, automaton::VisiblyPushdownDPDA < >, string::LinearString < > > ( Result::result );

label::Label Result::result ( const automaton::RealTimeHeightDeterministicDPDA < > & automaton, const string::LinearString < > & string, const label::Label & failLabel ) {
	std::tuple < bool, label::Label, std::set < unsigned >, std::deque < alphabet::Symbol > > res = Run::calculateState ( automaton, string );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res );

	return failLabel;
}

auto ResultRealTimeHeightDeterministicDPDALinearString = Result::RegistratorWrapper < label::Label, automaton::RealTimeHeightDeterministicDPDA < >, string::LinearString < > > ( Result::result );

label::Label Result::result ( const automaton::DPDA < > & automaton, const string::LinearString < > & string, const label::Label & failLabel ) {
	std::tuple < bool, label::Label, std::set < unsigned >, std::deque < alphabet::Symbol > > res = Run::calculateState ( automaton, string );

	if ( std::get < 0 > ( res ) ) return std::get < 1 > ( res );

	return failLabel;
}

auto ResultDPDALinearString = Result::RegistratorWrapper < label::Label, automaton::DPDA < >, string::LinearString < > > ( Result::result );

} /* namespace run */

} /* namespace automaton */
